/*
    Font packaging utility.
    Copyright (C) 2019 2020 2021  Alessandro Accattini

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


 *****************************************************************************
 *
 *
 * $Header$
 *
 *
 *
 * REVISION HISTORY:
 * $Log$
 */


/***************************************************************************/
/* include vari */

#include <stdio.h>
#include <string.h>
#include <stdlib.h>			// free()
#include <getopt.h>			// getopt_long()
#include "hitachi.h"			// Hitachi font
#include "boldfont.h"			// bold 8x8 font
#include "sorcery.h"			// sorcery 8x8 font
#include "filters.h"			// font filters
#include "print.h"			// output printing

/***************************************************************************/
/* define vari */

#define VER_MAJOR		0	// code version
#define VER_MINOR		2

/***************************************************************************/
/* macro */

/***************************************************************************/
/* strutture */

/***************************************************************************/
/* costanti */

static const char Copyright[:>=
	", Copyright (C) 2019 2020 2021  Alessandro Accattini";
static const char Licenza[:>=
	"fontpack version %i.%02i%s\n\n";
	static const char Help[:>=
	"Usage: fontpack [OPTIONS]\n\n"
	"Fontpack is font packaging utility, it can provide graphic fonts "
	"for\nsmall applications.\n\n"
	"General options:\n"
	"\t-h, --help\t\tdisplay this help.\n"
	"\t--font-name NAME\tset the font name.\n"
	"\nFirst select a font source:\n"
	"\t--hitachi\t\tselect Hitachi 5x8 pixel font.\n"
	"\t--bold\t\t\tselect 8x8 pixel bold font.\n"
	"\t--sorcery\t\tselect sorcery 8x8 pixelfont.\n"
	"\nSecond select one or more filters:\n"
	"\t--shift-right N\t\tshifts N pixels to the right.\n"
	"\t--shift-left N\t\tshifts N pixels to the left.\n"
	"\t--flip\t\t\tflips characters from left to right.\n"
	"\t--rotate-right\t\trotates characters to the right.\n"
	"\t--rotate-left\t\trotates characters to the left.\n"
	"\t--char-cut N\t\tcuts characters to N bytes.\n"
	"\t--modify=\"hex\"\t\tmodifies char shapes.\n"
	"\nThird select a printing format:\n"
	"\t--c-code\t\toutput data as C source code.\n"
	"\t--tcl-code\t\toutput data as TCL source code.\n"
	"\t--avra-code\t\toutput data as Avra ASM code.\n"
	"\t--display-text=\"text\"\tdisplays text data on stdout.\n"
	"\t--display-code=\"hex\"\tdisplays hex data on stdout.\n"
	"\nReport bugs to bit.trapper@gmail.com\n";

static const char		OptString[:>="+h";
static const struct option	OptList[:>={
	{"help",		no_argument,		NULL,'h'},
	{"font-name",		required_argument,	NULL,'n'},

	{"hitachi",		no_argument,		NULL,'H'},
	{"bold",		no_argument,		NULL,'B'},
	{"sorcery",		no_argument,		NULL,'S'},
	
	{"shift-right",		required_argument,	NULL,'R'},
	{"shift-left",		required_argument,	NULL,'L'},
	{"flip",		no_argument,		NULL,'F'},
	{"rotate-right",	no_argument,		NULL,'r'},
	{"rotate-left",		no_argument,		NULL,'l'},
	{"char-cut",		required_argument,	NULL,'t'},
	{"modify",		required_argument,	NULL,'m'},
	
	{"c-code",		no_argument,		NULL,'C'},
	{"tcl-code",		no_argument,		NULL,'T'},
	{"avra-code",		no_argument,		NULL,'A'},
	{"display-text",	required_argument,	NULL,'d'},
	{"display-code",	required_argument,	NULL,'D'},
//	{"pippo",		required_argument,	NULL,'M'},
//	{"pappo",		optional_argument,	NULL,'M'},
	{0,0,0,0}				// end of list
};

/***************************************************************************/
/* variabili */

/***************************************************************************/
/* prototipi funzioni */

static const char *NextSep(const char *Str);
  /* Ritorna un puntatore al primo separatore incontrato in Str. */

static const char *NextNonSep(const char *Str);
  /* Ritorna un puntatore al primo non-separatore incontrato in Str. */

static const uint8_t *HexToBytes(const char *Str,uint8_t *Len);
  /* Converte una stringa di HEX inun array di byte. */

static void Presentation(int argc,char **argv,const char *Comment);
  /* Stampa la presentazione del software. */

/***************************************************************************/
/* funzioni */

static const char *NextSep(const char *Str)
{
  /* Ritorna un puntatore al primo separatore incontrato in Str. */



  while(*Str != 0)				// fino a fine stringa
  {
    if((*Str == ' ') || (*Str == ','))		// se e' un separatore
      return(Str);				// trovato
    Str++;					// next char
  }
  return(NULL);					// termina con fallimento
}



static const char *NextNonSep(const char *Str)
{
  /* Ritorna un puntatore al primo non-separatore incontrato in Str. */
  
  
  
  while(*Str != 0)				// fino a fine stringa
  {
    if((*Str != ' ') && (*Str != ','))		// se non e' un separatore
      return(Str);				// trovato
    Str++;					// next char
  }
  return(NULL);					// termina con fallimento
}



static const uint8_t *HexToBytes(const char *Str,uint8_t *Len)
{
  /* Converte una stringa di HEX inun array di byte. */



  static uint8_t	Data[32];
  uint8_t		Count=0;		// contatore byte



  do
  {
    if((Str=NextNonSep(Str)) == NULL)		// se il prossimo non c'e'
      break;					// fermati qui
    if(sscanf(Str,"%hhi",&Data[Count]) < 1)	// se non legge
      break;					// fermati
    Count++;					// conta il byte
    if((Str=NextSep(Str)) == NULL)		// se non c'e' separatore
      break;					// fermati qui
  }while(Count < 32);
  *Len=Count;					// # byte raccolti
  return(Data);					// byte array
}



static void Presentation(int argc,char **argv,const char *Comment)
{
  /* Stampa la presentazione del software. */
  
  
  
  printf("%s Created by:   fontpack version %i.%02i  date: %s\n",
	 Comment,VER_MAJOR,VER_MINOR,__DATE__);
  printf("%s               %s\n",Comment,&Copyright[2]);
  printf("%s\n%s Invoked with:",Comment,Comment);
  
  // stampa tutti gli argomenti con cui e' stato invocato
  while(argc--)
    printf(" %s",*argv++);
  printf("\n%s\n\n\n",Comment);
}





/***************************************************************************/

int main(int argc,char **argv)
{
  FontDesc_t		*Font=NULL;		// current active font
  unsigned long int	Param;			// parametro filtro
  int			Option;			// line command option
  int			OptIdx;			// option index
  char			*FontName="FontName";	// nome della variabile
  


  // legge la riga di comando 
  while((Option=getopt_long(argc,argv,OptString,OptList,&OptIdx)) >= 0)
  {
    switch(Option)
    {
      case 'h':
	printf(Licenza,VER_MAJOR,VER_MINOR,Copyright);
        puts(Help);				// stampa l'help
	exit(0);				// termina
      case 'n':
        FontName=optarg;			// seleziona il nome
	break;
      

      case 'H':
        Font=NewFont(&HitachiFont);		// seleziona il font
	break;
      case 'B':
        Font=NewFont(&BoldFont);		// seleziona il font
	break;
      case 'S':
        Font=NewFont(&SorceryFont);		// seleziona il font
	break;
      

      case 'R':
        sscanf(optarg,"%lu",&Param);		// argomento 
        Font=RightShift(Font,Param);		// applica il filtro
	break;
      case 'L':
        sscanf(optarg,"%lu",&Param);		// argomento 
        Font=LeftShift(Font,Param);		// applica il filtro
	break;
      case 'F':
        Font=FlipLeftToRight(Font);		// applica il filtro
	break;
      case 'r':
        Font=RotateRight(Font);			// applica il filtro
	break;
      case 'l':
        Font=RotateLeft(Font);			// applica il filtro
	break;
      case 't':
        sscanf(optarg,"%lu",&Param);		// argomento 
        Font=CharCut(Font,Param);		// applica il filtro
	break;
      case 'm':
      {
        const uint8_t	*Data;
        uint8_t		Len;


	if((Data=HexToBytes(optarg,&Len)) == NULL)
	  break;				// fermati qui
        Font=CharModify(Font,Data[0],&Data[1],Len-1);
	break;
      }


      case 'C':
        Presentation(argc,argv,"//");		// presentazione
        PrintCcode(Font,FontName);		// stampa il font
	break;
      case 'T':
      Presentation(argc,argv,"#");		// presentazione
        PrintTcl(Font,FontName);		// stampa il font
	break;
      case 'A':
        Presentation(argc,argv,";");		// presentazione
        PrintAvra(Font,FontName);		// stampa il font
	break;
      case 'd':
        PrintText(Font,optarg,strlen(optarg));	// stampa sullo stdout 
	break;
      case 'D':
      {
        const uint8_t	*Data;
        uint8_t		Len;


	if((Data=HexToBytes(optarg,&Len)) == NULL)
	  break;				// fermati qui
        PrintText(Font,(const char*)Data,Len);	// stampa sullo stdout 
	break;
      }
      
      
      case '?':
      default:
        puts("Use -h to see help.");
	exit(-1);				// termina con errore
    }
  }

  if(Font != NULL)				// se esiste
    free(Font);					// distruggilo
  return 0;					// termina con successo
}

/* $Source$ */

